/*
    libcentroidal_profile provides functions to calculate centroidal profiles
    of regions and match them against one another.
    Copyright (C) 2004  Sebastian Hans <hanss@in.tum.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <centroidal_profile.h>

#include <stdlib.h> /* malloc, free */

static inline cp_profile_t *cp_generate_profile_from_image(IplImage *image,
		long num_points)
{
	IplImage *cloned_image;
	cp_profile_t *profile;

	if (!(cloned_image = cvCloneImage(image)))
		return NULL;

	profile = cp_create_profile(cloned_image, num_points);

	cvReleaseImage(&cloned_image);

	return profile;
}

cp_model_t *cp_create_model(IplImage *image, long id, char *name,
		long num_points)
{
	return cp_create_model_with_user_data(image, id, name, num_points,
			NULL);
}

cp_model_t *cp_create_model_with_user_data(IplImage *image, long id, char *name,
		long num_points, void *user_data)
{
	cp_model_t *model;

	if (!(model = malloc(sizeof(cp_model_t))))
		return NULL;

	if (!(model->profile = cp_generate_profile_from_image(image,
					num_points)))
	{
		free(model);
		return NULL;
	}

	model->image = image;
	model->id = id;
	model->name = name;
	model->user_data = user_data;
	model->prev = model->next = NULL;

	return model;
}

int cp_set_num_points_for_model(cp_model_t *model, long num_points)
{
	IplImage *cloned_image;
	int success;

	if (!(cloned_image = cvCloneImage(model->image)))
		return 0;

	success = cp_recalculate_profile(model->profile, cloned_image,
			num_points);

	cvReleaseImage(&cloned_image);

	return success;
}

void cp_destroy_model(cp_model_t *model, int free_name, int free_user_data)
{
	if (!model)
		return;
	cvReleaseImage(&model->image);
	cp_destroy_profile(model->profile);
	if (free_name && model->name)
		free(model->name);
	if (free_user_data && model->user_data)
		free(model->user_data);
	free(model);
}
